/*
 * Copyright (c) 2023 DTB <trinity@trinity.moe>
 * Copyright (c) 2023 Marceline Cramer <mars@tebibyte.media>
 * SPDX-License-Identifier: AGPL-3.0-or-later
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see https://www.gnu.org/licenses/.
 */

#include <ctype.h>
#include <stddef.h> /* NULL */
#include <stdio.h> /* fprintf(3) */
#include <stdlib.h> /* size_t, EXIT_FAILURE */
#include <string.h> /* strcmp(3) */
#include <sysexits.h> /* EX_USAGE */

char *program_name = "str";

static struct {
	char *name;
	int (*f)(int);
} ctypes[] = {
	{ "isalnum", isalnum },
	{ "isalpha", isalpha },
	{ "isblank", isblank },
	{ "iscntrl", iscntrl },
	{ "isdigit", isdigit },
	{ "isxdigit", isxdigit },
	{ "isgraph", isgraph },
	{ "islower", islower },
	{ "isprint", isprint },
	{ "ispunct", ispunct },
	{ "isspace", isspace },
	{ "isupper", isupper },
	{ NULL, NULL } /* marks end */
};

int usage(char *s){
	fprintf(stderr, "Usage: %s type string...\n", s);
	return EX_USAGE;
}

int main(int argc, char *argv[]){
	size_t ctype; /* selected from ctypes.h; index of ctype */
	int retval;

	if (argc < 3) { return usage(argv[0] == NULL ? program_name : argv[0]); }

	for ( /* iterate ctypes */
		ctype = 0;
		ctypes[ctype].f != NULL /* break at the end of ctypes */
			&& strcmp(argv[1], ctypes[ctype].name) != 0; /* break at match */
		++ctype
	);

	if (ctypes[ctype].f == NULL) { return usage(argv[0]); }

	/* iterate args */
	for (argv += 2, retval = EXIT_FAILURE; *argv != NULL; ++argv) {
		for (size_t i = 0; argv[0][i] != '\0'; ++i) { /* iterate arg bytes */
			/* First checks if argv[0][i] is valid ASCII; ctypes(3) don't
			 * handle non-ASCII. This is bad. */
			if(
				(unsigned char)argv[0][i] < 0x80 && !ctypes[ctype].f(argv[0][i])
			) { return EXIT_FAILURE; }
			else { retval = EXIT_SUCCESS; }
		}
	}

	return retval;
}
